#!/bin/bash

################################################################################
#
#	File:		bounce_licensed.command
#
#	Contains:	Shell script that stops the LicenseD service, removes the
#				client license database, then starts the service up again.
#
#				You must be logged in as an administrator before running this 
#				script. During the script's execution, you will be asked for 
#				an administrator password.
#
#	Copyright:	(C) 22013 by PACE Anti-Piracy, all rights reserved.
# 
################################################################################

#-------------------------------------------------------------------------------
# Initialize script globals
#-------------------------------------------------------------------------------
# Get the OS version
osVersion=`uname -r`
osMajorVersion=${osVersion%%.*}

# Assume no error for now
result=0

#-------------------------------------------------------------------------------
# If a file or folder exists at the specified path, then delete it. If it's a
# directory, then its entire contents is deleted.
#-------------------------------------------------------------------------------
deleteIfExists()
{
	if [[ -e "$1" ]]; then
		# If it's a directory, then make sure it contains no locked files
		if [[ -d "$1" ]]; then
			sudo chflags -R nouchg "$1"
		fi
		
		# Do the remove and check for an error.
		sudo /bin/rm -rf "$1"
		if [[ $? -ne 0 ]]; then
			echo "Unable to delete $1. FATAL ERROR!"
			exit 1
		fi
	fi
}

#-------------------------------------------------------------------------------
# If LicenseD is running, then stop it now. After stopping it, clean up by
# removing LicenseD's socket file if found.
#-------------------------------------------------------------------------------
unloadLicenseDManually()
{
	echo "Unloading license daemon as needed..."

	# If an instance of the daemon is currently running, unload it using launchctl.
	# We may have to wait for it to be gone.
	daemonLabel="com.paceap.eden.licensed"
	`sudo /bin/launchctl list | /usr/bin/grep -q "$daemonLabel"`
	if [[ $? -eq 0 ]]; then
		echo "Unloading license daemon..."
	
		# On 10.5 or later, we use the "remove" command since it tolerates
		# a missing or renamed plist file
		result=0
		if [[ $osMajorVersion -lt 9 ]]; then
			# The current system predates 10.5. First see if we even have the launchd folder.
			launchDaemonsDir="/Library/LaunchDaemons"
			if [[ -d "$launchDaemonsDir" ]]; then
				echo "Unloading LicenseD from launchctl..."
				`sudo /bin/launchctl unload "${launchDaemonsDir}/${daemonLabel}.plist" > /dev/null`
				result=$?
			else
				echo "The \"$launchDaemonsDir\" directory does NOT exist. No need to try to unload LicenseD."
			fi
		else
			echo "Removing LicenseD from launchctl..."
			`sudo /bin/launchctl remove "$daemonLabel" > /dev/null`
			result=$?
		fi
	
		# Check the unload or remove result
		if [[ $result -ne 0 ]]; then
			fatalError "launchctl remove failed with error $result"
		fi
	
		# Double-check that the daemon was successfully removed. We check a certain number of 
		# times with a 1-second delay between checks.
		daemonStopped=0
		daemonCheckCount=0
		for (( daemonCheckCount=0 ; $daemonCheckCount != 60 ; daemonCheckCount=daemonCheckCount+1 ))
		do
			`sudo /bin/launchctl list | /usr/bin/grep -q "$daemonLabel"`
			if [[ $? -ne 0 ]]; then
				daemonStopped=1
				echo "The license daemon has stopped."
				break
			fi
	
			echo "Waiting for the license daemon to stop..."
			sleep 1			# give it more time
		done
	
		# If we LicenseD was not stopped, report the error and fail
		if [[ $daemonStopped -eq 0 ]]; then
			fatalError "LicenseD still found under launchctl control after $daemonCheckCount seconds."
		fi
	
		# LicenseD was successfully unloaded
		echo "License daemon was unloaded."
	else
		# LicenseD was not running
		echo "License daemon is not running under launchd. Nothing to do."
	fi
	
	# Clean out the sockets directory and remove it in case it has the wrong
	# owner or permissions.
	deleteIfExists "/var/tmp/com.paceap.eden.licensed"
	
	echo
}

#-------------------------------------------------------------------------------
# If this version of LicenseD has a script built into it that knows how to 
# unload it, then call the script. This allows the loading and unloading of 
# LicenseD to be centralized into the application package itself.
#
# If the unload script does not exist, then perform the unload manually.
#-------------------------------------------------------------------------------
unloadLicenseD()
{
	daemonDest="/Library/PrivilegedHelperTools"
	unloadScript="$daemonDest/licenseDaemon.app/Contents/Resources/licensed_unload.sh"
	if [[ -f "$unloadScript" ]]; then
		echo "Calling the license daemon unload script here: \"$unloadScript\""
		echo
		sudo "$unloadScript"
		result=$?
		if [[ $result -ne 0 ]]; then
			fatalError "The LicenseD unload script returned an error: $result"
		fi
	else
		# If get here, we couldn't find the unload script. Do a manual unload.
		echo "Could not find the license daemon unload script. Manually unloading instead..."
		echo
			
		# Unload LicenseD, if it's running, then clean up after it.
		unloadLicenseDManually
	fi
}

#-------------------------------------------------------------------------------
# Manually load LicenseD. This will only be called with earlier versions of
# LicenseD that do not contain the load script.
#
# Load LicenseD via launchctl if it's not already running.
#-------------------------------------------------------------------------------
loadLicenseDManually()
{
	echo "Checking to see if license daemon is running..."

	# See if we LicenseD is already running under launchd control
	`/bin/launchctl list | /usr/bin/grep -q "$daemonLabel"`
	if [[ $? -ne 0 ]]; then
		# Check to see if the LicenseD plist file exists
		if [ ! -f "$daemonConfigPath" ]; then
			fatalError "Could not find the LicenseD plist. Maybe you didn't install License Support."
		fi
	
		# We need to load LicenseD under launchd
		echo "Loading license daemon..."
		`/bin/launchctl load "$daemonConfigPath" > /dev/null`
		result=$?
		if [[ $result -ne 0 ]]; then
			fatalError "Attempt to load $daemonConfigPath with launchctl failed with error $result."
		fi
		
		# Double-check that the daemon was successfully loaded. We check a certain number of 
		# times with a 1-second delay between checks.
		daemonStarted=0
		daemonCheckCount=0
		for (( daemonCheckCount=0 ; $daemonCheckCount != 5 ; daemonCheckCount=daemonCheckCount+1 ))
		do
			`/bin/launchctl list | /usr/bin/grep -q "$daemonLabel"`
			result=$?
			if [[ $result -eq 0 ]]; then
				daemonStarted=1
				echo "The license daemon has started."
				break
			fi
	
			echo "Waiting for the license daemon to start..."
			sleep 1			# give it more time
		done
	
		# If we LicenseD was not started, report the error and fail
		if [[ $daemonStarted -eq 0 ]]; then
			fatalError "LicenseD was not found under launchctl control after $daemonCheckCount seconds. Last error: $result."
		fi
			
		# If we get here, LicenseD was successfully loaded.
		echo "License daemon was successfully loaded under launchctl."
	else
		echo "License daemon is already running under launchctl. Nothing to do."
	fi
	
	echo
}

#-------------------------------------------------------------------------------
# If this version of LicenseD has a script built into it that knows how to 
# load it, then call the script. This allows the loading and unloading of 
# LicenseD to be centralized into the application package itself.
#
# If the load script does not exist, then perform the load manually.
#-------------------------------------------------------------------------------
loadLicenseD()
{
	daemonDest="/Library/PrivilegedHelperTools"
	loadScript="$daemonDest/licenseDaemon.app/Contents/Resources/licensed_load.sh"
	if [[ -f "$loadScript" ]]; then
		echo "Calling the license daemon load script here: \"$loadScript\""
		echo
		sudo "$loadScript"
		result=$?
		if [[ $result -ne 0 ]]; then
			fatalError "The LicenseD load script returned an error: $result"
		fi
	else
		# If get here, we couldn't find the load script. Do a manual load.
		echo "Could not find the license daemon load script. Manually loading instead..."
		echo
			
		# Load LicenseD manually
		loadLicenseDManually
	fi
}


#-------------------------------------------------------------------------------
# Script starts here
#-------------------------------------------------------------------------------
echo
echo "Restart the PACE License Services daemon"
echo 

if [ $osMajorVersion -lt 8 ]; then
	# Panther or below
	echo "This script can only run on MacOS 10.4 or later"
	echo "MacOS is a registered trademark of Apple Computer, Inc."
	exit 1
fi

# We don't want to uninstall our Xcode plug-ins while Xcode is running.
# So if Xcode is running, tell the user to quit it, then return an error.
foundXcode=`ps -axcopid,command | grep "iLok License Manager"`
if [[ ! -z $foundXcode ]]; then
	echo "You cannot run this uninstall script while the iLok License Manager is running."
	echo "Please quit the iLok License Manager and try again."
	exit 1
fi

echo "You must be logged in to an administrator account before you can run this script."
echo 
printf "Are you sure you want to restart License Services (y/n)? "
read query
echo

case $query in
	y | Y)
	sudo -p "Please enter your administrator password: " -v
	if [ $? = 1 ] ; then
		echo 
		echo "Authentication failed, or this user is not an Administrator."
		echo "Please log in as an Administrator and try again."
	else
		# Unload LicenseD by script (if found) or manually if not.
		unloadLicenseD

		# Delete the client license database file
		echo "Deleting the client side license database..."
		deleteIfExists "/Library/Preferences/com.paceap.eden.clientdb.v1.01.sdb"
		
		# Unload LicenseD by script (if found) or manually if not.
		loadLicenseD

		sudo -k
		echo
		echo "LicenseD has been bounced successfully."
		echo
	fi
	;;
	*)
		echo "LicenseD has not been bounced."
		echo
	;;
esac

exit 0
